<?php
namespace Financeiro\Logic\ContasReceber\Cnab\Remessa;

use \Financeiro\Entities\Cnabrec;
use Financeiro\Entities\Emprbanc;
use Geral\Entities\Contcorr;

abstract class GeraCnab{

	const CODIGO_COMPENSACAO_SANTANDER   = "033";
    const CODIGO_COMPESACAO_BRADESCO     = "237";
    const CODIGO_COMPESACAO_ITAU         = "341";
    const CODIGO_COMPESACAO_BRASIL       = "001";
    const CODIGO_COMPESACAO_CAIXA        = "104";

	public static function factory(Cnabrec $remessa)
    {
        $codigoCompensacao = $remessa->getReducont()->getAgencia()->getBanco()->getCdbanccnab();
        if ($codigoCompensacao === self::CODIGO_COMPENSACAO_SANTANDER) {
            return new GeraCnabSantander($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_BRADESCO) {
            return new GeraCnabBradesco($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_ITAU) {
            return new GeraCnabItau($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_BRASIL) {
            return new GeraCnabBrasil($remessa);
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_CAIXA) {
            return new GeraCnabCaixa($remessa);
        }

        return null;
    }

    protected function getIdContRec($cdreducont, $cdbancocnab, $cdempresa){
        /** @var $contcorr Contcorr */
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        $resultado = '';
        if($cdbancocnab and $cdbancocnab = '001'){
            /** @var $emprbanc Emprbanc */
            $emprbanc = Emprbanc::findOneByCdempresaAndCdbanco($cdempresa, $cdbancocnab);
        }

        if($contcorr and $contcorr->getCdidreccontcorr()){
            $resultado =  $contcorr->getCdidreccontcorr();
        }
        else{
            if($cdbancocnab != '001')
                $emprbanc = Emprbanc::findOneByCdempresaAndCdbanco($cdempresa, $cdbancocnab);
            if($emprbanc and $emprbanc->getCdempbancrec())
                $resultado = $emprbanc->getCdempbancrec();
        }

        return $resultado;
    }

	public abstract function geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL);
	public abstract function geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL);
	public abstract function geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
												  $nrcnabrec, $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero,
												  $idtipocobranca, $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec,
												  $valorpemulbrec, $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol,
                                                  $qtdiaprotbol);
	public abstract function geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec);
	public abstract function geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec);
	public abstract function GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco);
	public abstract function GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco);
    public abstract function geraTransacao($caminho, $nmarquivo, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
                                           $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
                                           $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
                                           $nrinsjurclie, $cdcliente, $cdins2cobbol);
}											 